import { Post, Tag, LogEntry, LogType } from "../type/generic";
import { Scrapper } from "../class/Scrapper";
import { getPageContents } from "../helper/requestManager";
import * as cheerio from 'cheerio';

export class InkBunny extends Scrapper {

    constructor() {
        // Set the domain base of the current Scrapper as "rule34.xxx"
        super("https://inkbunny.net");
    }

    /**
     * Get a list of posts from the mentioned page
     */
    public async getPostsFromPage(url: string): Promise<Array<string>> {

        // Check if the provided link is valid
        if (!this.checkURLBase(url)) {
            throw new Error(`Invalid url provided`);
        }

        // Initialize the page contents here
        let pageContents: string = null;

        // Send out the request to grab the contents of the post
        try {
            if (this.verbose) {
                console.error(`Sniffing page...`);
            }
            // Send out the initial Axios request to fetch the data from the page
            await getPageContents(url)
                .then(request => {
                    if (request.status < 200 || request.status > 299) {
                        this.logs.push({
                            msg: `Invalid response code[${request.status}]`,
                            type: LogType.ERROR,
                            err: null,
                            data: null,
                            ts: new Date()
                        });
                        throw new Error(`Invalid response code[${request.status}]`);
                    }

                    pageContents = (request.data as string);
                })
        } catch (err) {
            // "Handle" the error so that it's in the above .catch
            this.logs.push({
                msg: `[Error]::getPostsFromPage::`,
                type: LogType.ERROR,
                err: (err as Error),
                data: null,
                ts: new Date()
            });
            throw err;
        }

        // Process the page's posts with cheerio
        const $ = cheerio.load((pageContents as string));

        // Define the post List
        const postList: Array<string> = [];

        // Workaround I guess
        let self = this;

        // Go through all of the posts
        $(`.widget_imageFromSubmission a`).each(function () {
            const href = $(this).attr(`href`);
            if (`${href}`.length >= 4)
                postList.push(`${self.domain}/${href}`);
        });

        if (this.verbose) {
            console.error(`Found ${postList.length} posts`);
        }

        return postList;
    }


}